-- --------------------------------------------------------- --
--	PSG test bench											 --
-- ========================================================= --
--	Copyright (c)2006 t.hara								 --
-- --------------------------------------------------------- --

library	ieee;
use		ieee.std_logic_1164.all;
use		ieee.std_logic_unsigned.all;

entity tb is
end tb;

architecture behavior of tb is

	-- test target
	component psg is
	  port(
		clk21m  : in std_logic;
		reset   : in std_logic;
		clkena  : in std_logic;
		req		: in std_logic;
		ack		: out std_logic;
		wrt		: in std_logic;
		adr		: in std_logic_vector(15 downto 0);
		dbi		: out std_logic_vector(7 downto 0);
		dbo		: in std_logic_vector(7 downto 0);

		joya	: inout std_logic_vector(5 downto 0);
		stra	: out std_logic;
		joyb	: inout std_logic_vector(5 downto 0);
		strb	: out std_logic;

		kana	: out std_logic;
		cmtin   : in std_logic;
		keymode : in std_logic;

		wave	: out std_logic_vector(7 downto 0)
	 );
	end component;

	-- reference model
	component org_psg is
	  port(
		clk21m	: in std_logic;
		reset	: in std_logic;
		clkena	: in std_logic;
		req		: in std_logic;
		ack		: out std_logic;
		wrt		: in std_logic;
		adr		: in std_logic_vector(15 downto 0);
		dbi		: out std_logic_vector(7 downto 0);
		dbo		: in std_logic_vector(7 downto 0);

		joya	: inout std_logic_vector(5 downto 0);
		stra	: out std_logic;
		joyb	: inout std_logic_vector(5 downto 0);
		strb	: out std_logic;

		kana	: out std_logic;
		cmtin	: in std_logic;
		keymode	: in std_logic;

		wave	: out std_logic_vector(7 downto 0)
	 );
	end component;

	constant CYCLE : time := 10 ns;

	type test_name is (
		preparation,
		tone_8cycle,
		noise_8cycle,
		tone_noise_8cycle,
		envelope_00,
		envelope_04,
		envelope_08,
		envelope_09,
		envelope_10,
		envelope_11,
		envelope_12,
		envelope_13,
		envelope_14,
		envelope_15,
		noise_envelope_00,
		noise_envelope_04,
		noise_envelope_08,
		noise_envelope_09,
		noise_envelope_10,
		noise_envelope_11,
		noise_envelope_12,
		noise_envelope_13,
		noise_envelope_14,
		noise_envelope_15
	);

	signal	tb_name : test_name := preparation;

	signal	clk21m	: std_logic;
	signal	reset	: std_logic;
	signal	clkena	: std_logic;
	signal	req		: std_logic;
	signal	wrt		: std_logic;
	signal	adr		: std_logic_vector(15 downto 0);
	signal	dbo		: std_logic_vector(7 downto 0);

	signal	cmtin	: std_logic;
	signal	keymode	: std_logic;

	signal	ack			: std_logic;
	signal	dbi			: std_logic_vector(7 downto 0);
	signal	joya		: std_logic_vector(5 downto 0);
	signal	stra		: std_logic;
	signal	joyb		: std_logic_vector(5 downto 0);
	signal	strb		: std_logic;
	signal	kana		: std_logic;
	signal	wave		: std_logic_vector(7 downto 0);

	signal	ref_ack		: std_logic;
	signal	ref_dbi		: std_logic_vector(7 downto 0);
	signal	ref_joya	: std_logic_vector(5 downto 0);
	signal	ref_stra	: std_logic;
	signal	ref_joyb	: std_logic_vector(5 downto 0);
	signal	ref_strb	: std_logic;
	signal	ref_kana	: std_logic;
	signal	ref_wave	: std_logic_vector(7 downto 0);

	signal	tb_end	: std_logic := '0';
begin

	--	instance
	u_target: psg
	port map(
		clk21m  => clk21m	,
		reset   => reset	,
		clkena  => clkena	,
		req		=> req		,
		ack		=> ack		,
		wrt		=> wrt		,
		adr		=> adr		,
		dbi		=> dbi		,
		dbo		=> dbo		,

		joya	=> joya		,
		stra	=> stra		,
		joyb	=> joyb		,
		strb	=> strb		,

		kana	=> kana		,
		cmtin   => cmtin	,
		keymode => keymode	,

		wave	=> wave		
	);

	-- reference model
	u_reference: org_psg
	port map(
		clk21m	=> clk21m		,
		reset	=> reset		,
		clkena	=> clkena		,
		req		=> req			,
		ack		=> ref_ack		,
		wrt		=> wrt			,
		adr		=> adr			,
		dbi		=> ref_dbi		,
		dbo		=> dbo			,

		joya	=> ref_joya		,
		stra	=> ref_stra		,
		joyb	=> ref_joyb		,
		strb	=> ref_strb		,

		kana	=> ref_kana		,
		cmtin	=> cmtin		,
		keymode	=> keymode		,

		wave	=> ref_wave		
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk21m <= '0';
		wait for 5 ns;
		clk21m <= '1';
		wait for 5 ns;
	end process;

	-- ----------------------------------------------------- --
	--	test bench											 --
	-- ----------------------------------------------------- --
	process
	begin
		-- init
		clkena	<= '1';
		req		<= '0';
		wrt		<= '0';
		adr		<= (others => '0');
		dbo		<= (others => '0');
		cmtin	<= '0';
		keymode	<= '0';

		-- reset
		reset <= '1';
		wait for CYCLE;
		reset <= '0';
		wait for CYCLE;

		tb_name	<= tone_8cycle;
		-- SOUND 7, &H3E
		adr		<= X"00A0";
		dbo		<= X"07";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"3E";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 0, 8
		adr		<= X"00A0";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 1, 0
		adr		<= X"00A0";
		dbo		<= X"01";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 8, 15
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0F";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 9, 0
		adr		<= X"00A0";
		dbo		<= X"09";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 10, 0
		adr		<= X"00A0";
		dbo		<= X"0A";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 1 ms;

		-- SOUND 8, 0
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 1 us;

		tb_name	<= noise_8cycle;
		-- SOUND 7, &H37
		adr		<= X"00A0";
		dbo		<= X"07";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"37";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 6, &H08
		adr		<= X"00A0";
		dbo		<= X"06";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 8, 15
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0F";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 500 us;

		-- SOUND 8, 0
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 1 us;

		tb_name	<= tone_noise_8cycle;
		-- SOUND 7, &H36
		adr		<= X"00A0";
		dbo		<= X"07";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"36";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 6, &H08
		adr		<= X"00A0";
		dbo		<= X"06";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 8, 15
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0F";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 500 us;

		-- SOUND 8, 0
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 1 us;

		tb_name	<= envelope_00;
		-- SOUND 7, &H3E
		adr		<= X"00A0";
		dbo		<= X"07";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"3E";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 8, 16
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"10";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 11, 8
		adr		<= X"00A0";
		dbo		<= X"0B";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 12, 0
		adr		<= X"00A0";
		dbo		<= X"0C";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 13, 0
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_04;
		-- SOUND 13, 4
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"04";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_08;
		-- SOUND 13, 8
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_09;
		-- SOUND 13, 9
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"09";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_10;
		-- SOUND 13, 10
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0A";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_11;
		-- SOUND 13, 11
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0B";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_12;
		-- SOUND 13, 12
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0C";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_13;
		-- SOUND 13, 13
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_14;
		-- SOUND 13, 14
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0E";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= envelope_15;
		-- SOUND 13, 15
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0F";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_00;
		-- SOUND 7, &H37
		adr		<= X"00A0";
		dbo		<= X"07";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"37";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 8, 16
		adr		<= X"00A0";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"10";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 11, 8
		adr		<= X"00A0";
		dbo		<= X"0B";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 12, 0
		adr		<= X"00A0";
		dbo		<= X"0C";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- SOUND 13, 0
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"00";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_04;
		-- SOUND 13, 4
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"04";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_08;
		-- SOUND 13, 8
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"08";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_09;
		-- SOUND 13, 9
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"09";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_10;
		-- SOUND 13, 10
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0A";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_11;
		-- SOUND 13, 11
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0B";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_12;
		-- SOUND 13, 12
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0C";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_13;
		-- SOUND 13, 13
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_14;
		-- SOUND 13, 14
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0E";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_name	<= noise_envelope_15;
		-- SOUND 13, 15
		adr		<= X"00A0";
		dbo		<= X"0D";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		adr		<= X"00A1";
		dbo		<= X"0F";
		wrt		<= '1';
		req		<= '1';
		wait for CYCLE;

		wrt		<= '0';
		req		<= '0';
		wait for CYCLE*3;

		-- wait
		wait for 100 us;

		tb_end <= '1';
		wait;
	end process;

end behavior;
